package Ightaria;


import java.awt.*;
import java.lang.Math;

import Globals.*;
import DrawTexts.*;
import DrawHills.*;
import CalculusAndVectors.*;





public class Hill extends Terrain{


/** Own name               */ String  name;                  
/** BackIndex              */ int     iXinHillsFamily; 
/** Navigation from Vessel 
    Hill from Vessel Azi   */ double hVp;


//===============================================================
// PaintHillAsStar parameters.
//---------------------------------------------------------------
Color SatelliteColorPhase1 = new Color( 255<<16 | 100<<8 | 100 );
Color SatelliteColorPhase2 = new Color( 100<<16 | 100<<8 | 255 );
int StarHalfPeriod=20;
int starXRange        = 3;
int starSizeAmplitude = 3;
int starSizeMin       = 1;
//===============================================================





//========================================================
//Visibility.
//--------------------------------------------------------
/** Flag                 */   boolean canBeAStar = false;

/** <pre> Flag.
    Hill has three appearences.
    They depend on closeness to driver and 
    direction of driver's head.
    The appearence can be considered as subobject of the hill.
</pre> */                     int drawAsCloseOrFarStar = 0; 
public static final           int DRAW_AS_NOT_STAR          = 0; //Hill, town, valley, ...
public static final           int DRAW_AS_CLOSE_STAR        = 1; 
public static final           int DRAW_AS_FAR_STAR          = 2; 

/** Hill is in range of scene, oVisualRange  */  boolean inSceneRange;

/** Hill is 
     1. star in front of
        driver.
     2. not star and in 
        range of scene   */   boolean visible;              
//--------------------------------------------------------
//Visibility.
//========================================================





//===========================================================================
//Measurements.
//---------------------------------------------------------------------------
/** Hill is too close to be measured         */  boolean measureTooClose;

//do1 Actually: do measurements. 
//"Display" is masked. Will be true if object in the view angle.
/** Labels ...          
    This parameter does not depend on driver's direction.
    In other words, Angleminer, measures hill by
    driver's order.
    */                        boolean displayMesurements = false;
/** Flag. Which appearence of star to measure: far or close.
    Flag values are:
    0, Close Hill or Close Star is to be measured.
    1, Only measured as far star:
                         */   public int measureFarOrClose = 0;
//===========================================================================


      
/** Construction flag    */   public boolean painted;

/** Run-time parameter:
    Euclidean Distance to
    start up.            */   public double startUpDistance;



   
public Hill(                    ) { super();                  }
public Hill(   boolean canBeAStar ) { this.canBeAStar = canBeAStar; }
public Hill(   SPad sp,               //Was = sPad[SPad.a_],
               SPad spOwn,
               int ScreenXL,    
               int ScreenYL,
               int center_x,
               int center_y,
               int funN, 
               boolean addFunctionToTerrainFunction
           ) { 
               super( sp,
                      spOwn,
                      ScreenXL,    
                      ScreenYL,
                      center_x,
                      center_y,
                      funN,
                      addFunctionToTerrainFunction
               );

} //Constructor end.







/** This is really PaintAsPartOfHills procedure. */

public void paintHill( boolean dispalyMoon, boolean displayAxes, 
                       int ExternalFrameNumber,
                       double oAltitudeViewAngle
                      ) {
  
       //ms("PaintAsPart Member=" + iXinHillsFamily + " FunN=" + funN );     

       if(ExternalFrameNumber > -1) frameNumber=ExternalFrameNumber;
       obsZ =           (int) oFFz;
       oFrontDistance = (int) ( oDistance * Math.cos(hOp));
       hOOx =           (int) (-oDistance * Math.sin(hOp));
       hOOy =           (int) oFrontDistance;
       hOOz =           posZ - obsZ;

       this.oAltitudeViewAngle = oAltitudeViewAngle;
       altitudeCos      = (int) (RotationModula * Math.cos(oAltitudeViewAngle));
       altitudeSin      = (int) (RotationModula * Math.sin(oAltitudeViewAngle));

       CosRotationAngle = (int) (RotationModula * Math.cos( oFp));
       //Hills psy = - obs psy: => sin -> - sin:
       SinRotationAngle = (int) (RotationModula * Math.sin(-oFp));

       //These are needed only to add a Terrain:
       //These must be the same as below:
       //hFFx = oFFMx - oDistance * Math.cos(hFp);
       //hFFy = oFFMy + oDistance * Math.sin(hFp);
       hFFx = oFFMx + hOOx * Math.cos(oFp) - hOOy * Math.sin(oFp);
       hFFy = oFFMy + hOOx * Math.sin(oFp) + hOOy * Math.cos(oFp);

       paintHillInternally( true,  //reconstruct.
                            false, 
                            dispalyMoon,
                            displayAxes
                          );

       if( displayMesurements && 0 == measureFarOrClose ) {
           int S2X = ObsToSX(hOOx, hOOy, 0);
           int S2Y = CurrScreenY;
           if(S2X > -ScreenXL && S2X < ScreenXL && S2Y > 0 && S2Y < ScreenYL) {
              Color c = SatelliteColorPhase1;
              if(StarHalfPeriod>0) {
                 FindMoonPhase(StarHalfPeriod);
                 if( phase_sig < 0 ) c = SatelliteColorPhase2;
              }
              g.setColor(c);
              g.drawString( name + "  Azi=" + hOp + " (Driver)",       S2X, S2Y);
              if( Scenario.level >= Scenario.levHFConqueror || 
                  Scenario.cheatEnabled ){
                  g.drawString( "Tau =" + oTau,                        S2X, S2Y + 14);
                  g.drawString( "Distance =" + oTau*HFold.hfRadius, S2X, S2Y + 28);
                  g.drawString( "StUp E-Distance =" + startUpDistance, S2X, S2Y + 42);
              } 
              //GS.con("Name must be drawn now:" + name);
           }
       }
       painted = true;

}       






public void paintHillAsStar(int ExternalFrameNumber) {

       //---------------------------------------------------
       //Checkers and Preparers
       //- - - - - - - - - - - - - - - - - - - - - - - - - - 
       double tangent = Math.tan(hOp) / tangentOfVision;
       if( .999 < Math.abs(tangent) ) return; 

       boolean loDisplayMesurements = false;
       double trickyAzi = hOp; 
       if( displayMesurements ) {
           switch(drawAsCloseOrFarStar) {
           case DRAW_AS_CLOSE_STAR:
                if( measureFarOrClose == 0 ) {
                    loDisplayMesurements = true; 
                }
                break;
           case DRAW_AS_FAR_STAR:   
                if( measureFarOrClose == 1 ) {
                        //In this package, Azi is always normalized.
                        //In particular, calculated to the minimal tau.
                        //So, "far star" will be 180 degrees back. Use this var:
                    if( hOp > 0. ) { 
                        trickyAzi -= Math.PI;
                    }else{
                        trickyAzi += Math.PI;
                    }
                    loDisplayMesurements = true;
                }
           }
       }
       //- - - - - - - - - - - - - - - - - - - - - - - - - - 
       //Checkers and Preparers
       //---------------------------------------------------



       //---------------------------------------------------
       //Now display picture of typical Ightaria star.
       //- - - - - - - - - - - - - - - - - - - - - - - - - - 
       if(ExternalFrameNumber>-1) frameNumber = ExternalFrameNumber;

       //do1 make less expensive definition:
       Color loPhase1 = new Color(255,255,255);
       Color loPhase2 = new Color(255,255,255);
       int   loStarXRange = starXRange;
       int   loStarSizeAmplitude = starSizeAmplitude;
       double loTau = 1.; //Assign something to pass Java compiler.

       //This interferes with other states and displays:
       //Do more accurate display:
       //sp.wrc("STAR LIFE DETECTOR");

       switch ( drawAsCloseOrFarStar ) {
       case DRAW_AS_CLOSE_STAR: loPhase1 = SatelliteColorPhase1;
                                loPhase2 = SatelliteColorPhase2;
                                loTau = oTau;
                                //This interferes with other states and displays:
                                //Do more accurate display:
                                //sp.wr("Close constellations ...");
                                //sp.wr("Not detected ...");
                                break;
       case DRAW_AS_FAR_STAR:   //sp.wr("Far constellations ...");
                                //sp.wr("Not detected ...");
                                loPhase1 = new Color(150,0,0);
                                loPhase2 = new Color(100,0,0);
                                if( 0 == (ExternalFrameNumber % 19) &&  3 == iXinHillsFamily ) {
                                    //Custom amlitude flashes requested by Star's Ruler:
                                    loPhase1 = new Color(255,0,255);
                                    loPhase2 = new Color(255,0,0);
                                }
                                loStarXRange = 0;
                                loStarSizeAmplitude = 0;
                                loTau = oTau - 2.*Math.PI;
       }

       Color c = loPhase1;

       //Now display picture of
       //typical Ightaria star.
       int x = center_x - (int) (tangent * ScreenXL);
       int xshift = 0;
       int diametr = 4;
       int diametr2;

       if(StarHalfPeriod>0) {
          FindMoonPhase(StarHalfPeriod);
          xshift = phase_sig * ( StarHalfPeriod - phase_sub * 2 ) * loStarXRange        / StarHalfPeriod;
          diametr =            ( StarHalfPeriod - phase_sub * 2 ) * loStarSizeAmplitude / StarHalfPeriod + 
                                 starSizeMin;

          if( phase_sig > 0 ) c = loPhase2;
       }
       
       diametr2 = diametr /2;
       g.setColor(c);
       if(diametr > 0) g.fillOval(x+xshift-diametr2, center_y-diametr2, diametr, diametr);

       //- - - - - - - - - - - - - - - - - - - - - - - - - - 
       //Now display picture of typical Ightaria star.
       //---------------------------------------------------



       //---------------------------------------------------
       //Now display measurements
       //- - - - - - - - - - - - - - - - - - - - - - - - - - 
       if( loDisplayMesurements ) {
           int wi = starXRange + starSizeAmplitude + 3;
           g.drawString( closeOrFarName() + "  Azi=" + trickyAzi + 
                             " ( from Driver ) ", x+wi, center_y);
           if( Scenario.level >= Scenario.levHFConqueror || Scenario.cheatEnabled ){
               //High level bonus:
               g.drawString( "Tau =" + loTau, x+wi, center_y + 14);
               g.drawString( "Tan(Tau) =" + Math.tan(loTau), x+wi, center_y + 28);
               g.drawString( "E-Distance =" + Math.tan(loTau)*HFold.hfRadius, x+wi, center_y - 42);
               g.drawString( "StUp E-Distance =" + startUpDistance, x+wi, center_y - 28);
               g.drawString( "Real Distance =" + Math.abs(loTau * HFold.hfRadius), x+wi, center_y - 14);
          } 
       }
       //- - - - - - - - - - - - - - - - - - - - - - - - - - 
       //Now display measurements
       //---------------------------------------------------

       if(ExternalFrameNumber < 0) frameNumber +=1; 

} //paintHillAsStar end.




//======================================================
// M e a s u r i n g 
//------------------------------------------------------
/** Flags toggler.
    Flags are: displayMesurements and
               measureFarOrClose.
*/
public void doMeasure(boolean trueOrFalse) {
       displayMesurements = trueOrFalse;
       if(displayMesurements) {
          measureFarOrClose = (measureFarOrClose+1)%2;
          if( 1 == measureFarOrClose ) {
              if( Scenario.level == Scenario.levJustBorn && 
                  !Scenario.cheatEnabled ) {
                  //Measurement of opposite stars is not possible for
                  //Baby Ightarians:
                  //"Drop" measurement:
                  measureFarOrClose = 0;
              }
          }
      }   
}
public double twinHVp;
public double twinTangentTau;
public double twinTau;
public double twinDerivative;
public String measuredName() {
       return(nameOfAppearance(measureFarOrClose));
}
public double twin_hVp(boolean doMoreMasures) {

       if( 0 == measureFarOrClose ) {
           twinHVp=hVp;
       }else{
           twinHVp=Vec.fixAzi(hVp-Math.PI);
       }
       if(!doMoreMasures) return twinHVp;

       if( 0 == measureFarOrClose  ) {
           twinTau = oTau;
       }else{
           twinTau = oTau - 2.*Math.PI;
       }

       twinTangentTau = Math.tan(twinTau);

       //Do cheat. "Measure" derivative:
       if( measureTooClose ) {
           //This value must not be used:
           twinDerivative = 1.e100;
       }else{
           twinDerivative = 1./(HFold.hfRadius * 
                                twinTangentTau);
           if( measureFarOrClose == 1 ) {
               twinDerivative = -twinDerivative;
           }
       }
       return twinHVp;

}
//------------------------------------------------------
// M e a s u r i n g 
//======================================================






//A p p e a r e n c e
/** Returns "catalog name" (not known name) when far
    to emphasize that far star is not well researched.
    Returns own name otherwise.
*/
public String closeOrFarName() {
       if( DRAW_AS_FAR_STAR == drawAsCloseOrFarStar) {
           return(nameOfAppearance(1));
       }else{
           return name;
       }
}










public String nameOfAppearance(int appearence) {
       if( 1 == appearence ) {
           return("Star OS" + (iXinHillsFamily*3)); 
                  //3 is an arbitrary factor for 
                  //Ightarians' catalog of unresearched stars.
        }else{
           return(name);
        }
}






} //Class end.
